
// fx - A compositing library representation.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef N_OPERATOR_INC_
#define N_OPERATOR_INC_

#include <map>

#include <GL/glew.h>

#include <boost/any.hpp>
#include <boost/shared_ptr.hpp>

#include <openpluginlib/pl/property.hpp>
#include <openpluginlib/pl/string.hpp>

#include <openmedialib/ml/audio.hpp>

#include <openeffectslib/fx/config.hpp>

namespace olib { namespace openeffectslib { namespace fx {

typedef olib::openmedialib::ml::audio_type_ptr	audio_buffer;

//
class FX_DECLSPEC n_operator
{
public:
	typedef openpluginlib::wstring			key_type;
	typedef boost::any						field_type;
	typedef std::map<key_type, field_type>	container;
	typedef container::reference			reference;
	typedef container::const_reference		const_reference;
	typedef container::iterator				iterator;
	typedef container::const_iterator		const_iterator;
	typedef container::size_type			size_type;
	typedef container::value_type			value_type;
	
public:
	typedef std::map<key_type, key_type>	param_container;
	typedef param_container::const_iterator	param_const_iterator;
	typedef param_container::value_type		param_value_type;

public:
	explicit n_operator( const container& fields, const param_container& params )
		: fields_( fields )
		, params_( params )
		, dirty_( true )
	{ }

	n_operator( const n_operator& other )
		: fields_( other.fields_ )
		, params_( other.params_ )
		, dirty_( other.dirty_ )
	{ }

	n_operator& operator=( const n_operator& other )
	{
		if( this != &other )
		{
			fields_ = other.fields_;
			params_ = other.params_;
			dirty_  = other.dirty_;
		}
		
		return *this;
	}

	void swap( n_operator& other )
	{
		fields_.swap( other.fields_ );
		params_.swap( other.params_ );
		std::swap( dirty_, other.dirty_ );
	}

	virtual ~n_operator( )
	{ }

public:
	template<class U>
	U value( const key_type& key )
	{ return *boost::any_cast<U>( &fields_[ key ] ); }
	
	template<class U>
	U* pointer( const key_type& key )
	{ return boost::any_cast<U>( &fields_[ key ] ); }

	template<class U>
	const U* const_pointer( const key_type& key ) const
	{
		const_iterator I = fields_.find( key );
		if( I == fields_.end( ) )
			return static_cast<const U*>( 0 );

		return boost::any_cast<U>( &I->second );
	}

public:
	param_const_iterator param_begin( ) const
	{ return params_.begin( ); }
	
	param_const_iterator param_end( ) const
	{ return params_.end( ); }
	
	openpluginlib::wstring type_name( const key_type& key )
	{
		param_const_iterator I = params_.find( key );
		if( I == params_.end( ) )
			return openpluginlib::wstring( );
		
		return I->second;
	}

public:
	virtual bool init( )
	{ return true; }

	virtual bool update_image( ) = 0;
	virtual bool update_audio( ) = 0;

	virtual bool dirty( ) const
	{ return dirty_; }

	virtual void set_dirty( bool dirty )
	{ dirty_ = dirty; }

public:
	virtual const audio_buffer ab( ) 	   const = 0;

	virtual void   bind_as_GL_texture( )   const = 0;
	virtual void   unbind_as_GL_texture( ) const = 0;
	virtual GLuint get_GL_texture( )	   const = 0;
	virtual GLenum get_GL_target( )	  	   const = 0;
	
	virtual int width( )  const  = 0;
	virtual int height( ) const  = 0;

	virtual float max_s( ) const = 0;
	virtual float max_t( ) const = 0;

private:
	container		fields_;
	param_container	params_;
	bool			dirty_;
};

typedef openpluginlib::value_property<long long> v_frame_position;

typedef boost::shared_ptr<n_operator>					n_operator_ptr;
typedef openpluginlib::value_property<n_operator_ptr>	v_node_ptr;

const openpluginlib::wstring v_node_typename			= L"SFNode";
const openpluginlib::wstring v_in_color_typename		= L"SFInColor";
const openpluginlib::wstring v_out_color_typename		= L"SFOutColor";
const openpluginlib::wstring v_in_audio_typename		= L"SFInAudio";
const openpluginlib::wstring v_out_audio_typename		= L"SFOutAudio";
const openpluginlib::wstring v_frame_position_typename	= L"SFOutAudio";

} } }

#endif
