
// openpluginlib - An plugin interface to openlibraries.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef SHADER_MANAGER_INC_
#define SHADER_MANAGER_INC_

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <cassert>

#if _MSC_VER >= 1400
#include <hash_map>
#else
#include <map>
#endif

#include <boost/filesystem/path.hpp>

#include <GL/glew.h>

#if defined ( HAVE_CG_RUNTIME )
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#endif

#include <openpluginlib/pl/config.hpp>
#include <openpluginlib/pl/property.hpp>

namespace olib { namespace openpluginlib {

typedef std::basic_string<GLcharARB> shader_string;

class OPENPLUGINLIB_DECLSPEC GLSL_shader_object
{
public:
		typedef wstring key_type;
		
private:
#if _MSC_VER >= 1400
		typedef stdext::hash_map<key_type, GLint>	uniform_container;
#else
		typedef std::map<key_type, GLint>			uniform_container;
#endif
		typedef uniform_container::value_type value_type;	

public:
	explicit GLSL_shader_object( )
		: program_( 0 )
		, vertex_( 0 )
		, fragment_( 0 )
		, valid_( false )
	{ }

public:
	GLhandleARB& program( )			{ return program_; }
	GLhandleARB  program( )  const	{ return program_; }
	GLhandleARB& vertex( )			{ return vertex_; }
	GLhandleARB  vertex( )	 const	{ return vertex_; }
	GLhandleARB& fragment( )		{ return fragment_; }
	GLhandleARB  fragment( ) const	{ return fragment_; }
	bool&		 valid( )			{ return valid_; }
	bool		 valid( )	 const  { return valid_; }

public:
	bool	add_uniform( const key_type& uni );
	GLint	find_uniform( const key_type& uni );

private:
	GLhandleARB program_;
	GLhandleARB vertex_;
	GLhandleARB fragment_;
	bool valid_;
	
private:
	uniform_container uni_;
};

#if defined ( HAVE_CG_RUNTIME )
struct OPENPLUGINLIB_DECLSPEC Cg_shader_object
{
	explicit Cg_shader_object( )
		: vert( 0 )
		, frag( 0 )
		, v_profile( static_cast<CGprofile>( 0 ) )
		, f_profile( static_cast<CGprofile>( 0 ) )
		, valid( false )
	{ }
	
	CGprogram vert, frag;
	CGprofile v_profile, f_profile;
	bool valid;
};
#endif

class OPENPLUGINLIB_DECLSPEC shader_manager
{
private:
	static shader_manager* revive_if_dead( )
	{
		void* p = new char [ sizeof( shader_manager ) ];

		new( p ) shader_manager;
		
		return static_cast<shader_manager*>( p );
	}

	static void destroy( void )
	{
		instance_->~shader_manager( );

		delete[ ] ( ( char* ) instance_ );
	}

public:	
	static shader_manager& instance( )
	{
		if( !instance_ )
		{
			if( destroyed_ )
			{
				destroyed_     = false;
				was_destroyed_ = true;
			}

			instance_ = revive_if_dead( );

			if( !was_destroyed_ )
				std::atexit( annihilate );
		}
		
		return *instance_;
	}

	static void annihilate( )
	{
		if( instance_ )
		{
			assert( !destroyed_ );
			destroy( );
			destroyed_ = true;
			instance_ = 0;
		}
	}
	
public:
	shader_string	open_shader_file( const boost::filesystem::path& path, bool prefix = true );
	bool			create_GLSL_shader( const shader_string& vertex, const shader_string& fragment, GLSL_shader_object& shader );

#if defined ( HAVE_CG_RUNTIME )
	CGcontext get_Cg_context( ) const
	{ return cg_context_; }
	
	bool create_Cg_shader( const std::string& vertex, const std::string& fragment, Cg_shader_object& shader );
#endif

public:
	bool bind_GL_uniform( GLhandleARB program, v_float& v, const wstring& name ) const;
	bool bind_GL_uniform( GLhandleARB program, v_vec4f& v, const wstring& name ) const;
	bool bind_GL_uniform( GLhandleARB program, v_matrix4f& v, const wstring& name ) const;

private:
	explicit shader_manager( );
	~shader_manager( );
	shader_manager( const shader_manager& );
	shader_manager& operator=( const shader_manager& );

private:
#if defined ( HAVE_CG_RUNTIME )
	CGcontext cg_context_;
#endif
	static shader_manager* instance_;
	static bool destroyed_;
	static bool was_destroyed_;
};

OPENPLUGINLIB_DECLSPEC shader_manager& shader_manager_instance( );

} }

#endif
